@@ -19,17 +19,13 @@
  */
 
 #include <libglom/connectionpool_backends/postgres_self.h>
+#include <libglom/connectionpool.h>
 #include <libglom/utils.h>
+#include <libglom/db_utils.h>
 #include <libglom/spawn_with_feedback.h>
-#include <giomm/file.h>
+#include <giomm.h>
 #include <glib/gstdio.h> // For g_remove
 
-#include <glibmm/convert.h>
-#include <glibmm/miscutils.h>
-#include <glibmm/stringutils.h>
-#include <glibmm/regex.h>
-#include <glibmm/main.h>
-#include <glibmm/shell.h>
 #include <glibmm/i18n.h>
 
 #include <libglom/gst-package.h>
@@ -97,10 +93,6 @@ namespace ConnectionPoolBackends
 "# IPv6 local connections:\n" \
 "host    all         all         ::1/128               md5\n"
 
-#define PORT_POSTGRESQL_SELF_HOSTED_START 5433
-#define PORT_POSTGRESQL_SELF_HOSTED_END 5500
-
-
 #define DEFAULT_CONFIG_PG_HBA_REMOTE_8p3 \
 DEFAULT_CONFIG_PG_HBA_LOCAL_8p3 \
 DEFAULT_CONFIG_PG_HBA_REMOTE_EXTRA
@@ -109,12 +101,12 @@ DEFAULT_CONFIG_PG_HBA_REMOTE_EXTRA
 DEFAULT_CONFIG_PG_HBA_LOCAL_8p3 \
 DEFAULT_CONFIG_PG_HBA_REMOTE_EXTRA
 
-#define PORT_POSTGRESQL_SELF_HOSTED_START 5433
-#define PORT_POSTGRESQL_SELF_HOSTED_END 5500
+static const int PORT_POSTGRESQL_SELF_HOSTED_START = 5433;
+static const int PORT_POSTGRESQL_SELF_HOSTED_END = 5500;
 
-static const char* DEFAULT_CONFIG_PG_IDENT = "";
-static const char* FILENAME_DATA = "data";
-static const char* FILENAME_BACKUP = "backup";
+static const char DEFAULT_CONFIG_PG_IDENT[] = "";
+static const char FILENAME_DATA[] = "data";
+static const char FILENAME_BACKUP[] = "backup";
 
 PostgresSelfHosted::PostgresSelfHosted()
 : m_network_shared(false)
@@ -176,7 +168,7 @@ Backend::InitErrors PostgresSelfHosted::initialize(const SlotProgress& slot_prog
 
   if(m_database_directory_uri.empty())
   {
-    std::cerr << "PostgresSelfHosted::initialize: m_database_directory_uri is empty." << std::endl;
+    std::cerr << G_STRFUNC << ": initialize: m_self_hosting_data_uri is empty." << std::endl;
     return INITERROR_OTHER;
   }
 
@@ -237,8 +229,8 @@ Backend::InitErrors PostgresSelfHosted::initialize(const SlotProgress& slot_prog
 
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
-  const std::string command_initdb = get_path_to_postgres_executable("initdb") + " -D " + Glib::shell_quote(dbdir_data) +
-                                        " -U " + initial_username + " --pwfile=" + Glib::shell_quote(temp_pwfile);
+  const std::string command_initdb = "\"" + get_path_to_postgres_executable("initdb") + "\" -D \"" + dbdir_data + "\"" +
+                                        " -U " + initial_username + " --pwfile=\"" + temp_pwfile + "\"";
 
   //Note that --pwfile takes the password from the first line of a file. It's an alternative to supplying it when prompted on stdin.
   const bool result = Glom::Spawn::execute_command_line_and_wait(command_initdb, slot_progress);
@@ -257,7 +249,7 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
 {
   Glib::ustring result;
 
-  const std::string command = get_path_to_postgres_executable("pg_ctl") + " --version";
+  const std::string command = "\"" + get_path_to_postgres_executable("pg_ctl") + "\" --version";
 
   //The first command does not return, but the second command can check whether it succeeded:
   std::string output;
@@ -274,7 +266,6 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
   //We want the characters at the end:
   const gchar VERSION_REGEX[] = "pg_ctl \\(PostgreSQL\\) (.*)";
 
-  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     regex = Glib::Regex::create(VERSION_REGEX);
@@ -284,15 +275,6 @@ Glib::ustring PostgresSelfHosted::get_postgresql_utils_version(const SlotProgres
     std::cerr << "Glom: Glib::Regex::create() failed: " << ex.what() << std::endl;
     return result;
   }
-  #else
-  std::auto_ptr<Glib::Error> ex;
-  regex = Glib::Regex::create(VERSION_REGEX, static_cast<Glib::RegexCompileFlags>(0), static_cast<Glib::RegexMatchFlags>(0), ex);
-  if(ex.get())
-  {
-    std::cerr << "Glom: Glib::Regex::create() failed: " << ex->what() << std::endl;
-    return result;
-  }
-  #endif
 
   if(!regex)
     return result;
@@ -326,7 +308,6 @@ float PostgresSelfHosted::get_postgresql_utils_version_as_number(const SlotProgr
   //We want the characters at the end:
   const gchar VERSION_REGEX[] = "^(\\d*)\\.(\\d*)";
 
-  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     regex = Glib::Regex::create(VERSION_REGEX);
@@ -336,15 +317,6 @@ float PostgresSelfHosted::get_postgresql_utils_version_as_number(const SlotProgr
     std::cerr << "Glom: Glib::Regex::create() failed: " << ex.what() << std::endl;
     return result;
   }
-  #else
-  std::auto_ptr<Glib::Error> ex;
-  regex = Glib::Regex::create(VERSION_REGEX, static_cast<Glib::RegexCompileFlags>(0), static_cast<Glib::RegexMatchFlags>(0), ex);
-  if(ex.get())
-  {
-    std::cerr << "Glom: Glib::Regex::create() failed: " << ex->what() << std::endl;
-    return result;
-  }
-  #endif
 
   if(!regex)
     return result;
@@ -400,7 +372,8 @@ Backend::StartupErrors PostgresSelfHosted::startup(const SlotProgress& slot_prog
 
   if(!(file_exists_uri(dbdir_uri)))
   {
-    std::cerr << "ConnectionPool::create_self_hosting(): The directory could not be found: " << dbdir_uri << std::endl;
+    //TODO: Use a return enum or exception so we can tell the user about this:
+    std::cerr << G_STRFUNC << ": The data directory could not be found: " << dbdir_uri << std::endl;
     return STARTUPERROR_FAILED_UNKNOWN_REASON;
   }
 
@@ -429,12 +402,12 @@ Backend::StartupErrors PostgresSelfHosted::startup(const SlotProgress& slot_prog
   //Attempt to ensure that the config files are correct:
   set_network_shared(slot_progress, m_network_shared); //Creates pg_hba.conf and pg_ident.conf
 
-
   const unsigned int available_port = discover_first_free_port(PORT_POSTGRESQL_SELF_HOSTED_START, PORT_POSTGRESQL_SELF_HOSTED_END);
-  //std::cout << "ConnectionPool::create_self_hosting():() : debug: Available port for self-hosting: " << available_port << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ":() : debug: Available port for self-hosting: " << available_port << std::endl;
   if(available_port == 0)
   {
-    std::cerr << "ConnectionPool::create_self_hosting(): No port was available between " << PORT_POSTGRESQL_SELF_HOSTED_START << " and " << PORT_POSTGRESQL_SELF_HOSTED_END << std::endl;
+    //TODO: Use a return enum or exception so we can tell the user about this:
+    std::cerr << G_STRFUNC << ": No port was available between " << PORT_POSTGRESQL_SELF_HOSTED_START << " and " << PORT_POSTGRESQL_SELF_HOSTED_END << std::endl;
     return STARTUPERROR_FAILED_UNKNOWN_REASON;
   }
 
@@ -446,21 +419,17 @@ Backend::StartupErrors PostgresSelfHosted::startup(const SlotProgress& slot_prog
   // -k specifies a directory to use for the socket. This must be writable by us.
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
-  const std::string dbdir_config = Glib::build_filename(dbdir, "config");
-  const std::string dbdir_hba = Glib::build_filename(dbdir_config, "pg_hba.conf");
-  const std::string dbdir_ident = Glib::build_filename(dbdir_config, "pg_ident.conf");
-  const std::string dbdir_pid = Glib::build_filename(dbdir, "pid");
-  const std::string command_postgres_start = get_path_to_postgres_executable("postgres") + " -D " + Glib::shell_quote(dbdir_data)
+  const std::string command_postgres_start = "\"" + get_path_to_postgres_executable("postgres") + "\" -D \"" + dbdir_data + "\" "
                                   + " -p " + port_as_text
                                   + " -i " //Equivalent to -h "*", which in turn is equivalent to listen_addresses in postgresql.conf. Listen to all IP addresses, so any client can connect (with a username+password)
-                                  + " -c hba_file=" + Glib::shell_quote(dbdir_hba)
-                                  + " -c ident_file=" + Glib::shell_quote(dbdir_ident)
-                                  + " -k " + Glib::shell_quote(dbdir)
-                                  + " --external_pid_file=" + Glib::shell_quote(dbdir_pid);
+                                  + " -c hba_file=\"" + dbdir + "/config/pg_hba.conf\""
+                                  + " -c ident_file=\"" + dbdir + "/config/pg_ident.conf\""
+                                  + " -k \"" + dbdir + "\""
+                                  + " --external_pid_file=\"" + dbdir + "/pid\"";
 
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
-  const std::string command_check_postgres_has_started = get_path_to_postgres_executable("pg_ctl") + " status -D " + Glib::shell_quote(dbdir_data);
+  const std::string command_check_postgres_has_started = "\"" + get_path_to_postgres_executable("pg_ctl") + "\" status -D \"" + dbdir_data + "\"";
 
   //For postgres 8.1, this is "postmaster is running".
   //For postgres 8.2, this is "server is running".
@@ -484,6 +453,44 @@ Backend::StartupErrors PostgresSelfHosted::startup(const SlotProgress& slot_prog
   return STARTUPERROR_NONE;
 }
 
+void PostgresSelfHosted::show_active_connections()
+{
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+      Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+  builder->select_add_field("*", "pg_stat_activity");
+  builder->select_add_target("pg_stat_activity");
+ 
+  Glib::RefPtr<Gnome::Gda::Connection> gda_connection = connect(m_saved_database_name, m_saved_username, m_saved_password);
+  if(!gda_connection)
+    std::cerr << G_STRFUNC << ": connection failed." << std::endl;
+  
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
+  if(!datamodel)
+    std::cerr << G_STRFUNC << ": pg_stat_activity SQL query failed." << std::endl;
+  
+  const int rows_count = datamodel->get_n_rows(); 
+  if(datamodel->get_n_rows() < 1)
+    std::cerr << G_STRFUNC << ": pg_stat_activity SQL query returned no rows." << std::endl;
+
+  std::cout << "Active connections according to a pg_stat_activity SQL query:" << std::endl;
+  const int cols_count = datamodel->get_n_columns();
+  for(int row = 0; row < rows_count; ++row)
+  {
+    for(int col = 0; col < cols_count; ++col)
+    {
+      if(col != 0)
+        std::cout << ", ";
+        
+      std::cout << datamodel->get_value_at(col, row).to_string();
+    }
+    
+    std::cout << std::endl;
+  }
+  
+  //Make sure that this connection does not stop a further attempt to stop the server.
+  gda_connection->close();
+}
+
 bool PostgresSelfHosted::cleanup(const SlotProgress& slot_progress)
 {
   // This seems to be called twice sometimes, so we don't assert here until
@@ -510,12 +517,22 @@ bool PostgresSelfHosted::cleanup(const SlotProgress& slot_progress)
   // TODO: Warn about connected clients on other computers? Warn those other users?
   // Make sure to use double quotes for the executable path, because the
   // CreateProcess() API used on Windows does not support single quotes.
-  const std::string command_postgres_stop = get_path_to_postgres_executable("pg_ctl") + " -D " + Glib::shell_quote(dbdir_data) + " stop -m fast";
+  const std::string command_postgres_stop = "\"" + get_path_to_postgres_executable("pg_ctl") + "\" -D \"" + dbdir_data + "\" stop -m fast";
   const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, slot_progress);
   if(!result)
   {
     std::cerr << "Error while attempting to stop self-hosting of the database. Trying again."  << std::endl;
-
+    
+    //Show open connections for debugging:
+    try
+    {
+      show_active_connections();
+    }
+    catch(const Glib::Error& ex)
+    {
+      std::cerr << G_STRFUNC << ": exception while trying to show active connections: " << ex.what() << std::endl;
+    }
+    
     //I've seen it fail when running under valgrind, and there are reports of failures in bug #420962.
     //Maybe it will help to try again:
     const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, slot_progress);
@@ -578,16 +595,14 @@ static bool on_timeout_delay(const Glib::RefPtr<Glib::MainLoop>& mainloop)
 }
 
 
-Glib::RefPtr<Gnome::Gda::Connection> PostgresSelfHosted::connect(const Glib::ustring& database, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<ExceptionConnection>& error)
+Glib::RefPtr<Gnome::Gda::Connection> PostgresSelfHosted::connect(const Glib::ustring& database, const Glib::ustring& username, const Glib::ustring& password)
 {
   if(!get_self_hosting_active())
   {
-    error.reset(new ExceptionConnection(ExceptionConnection::FAILURE_NO_BACKEND)); //TODO: But there is a backend. It's just not ready.
+    throw ExceptionConnection(ExceptionConnection::FAILURE_NO_BACKEND); //TODO: But there is a backend. It's just not ready.
     return Glib::RefPtr<Gnome::Gda::Connection>();
   }
 
-  std::auto_ptr<ExceptionConnection> ex;
-
   Glib::RefPtr<Gnome::Gda::Connection> result;
   bool keep_trying = true;
   guint count_retries = 0;
@@ -595,45 +610,58 @@ Glib::RefPtr<Gnome::Gda::Connection> PostgresSelfHosted::connect(const Glib::ust
   const guint MAX_RETRIES_EVER = 60; /* seconds */
   while(keep_trying)
   {
-    result = attempt_connect(port_as_string(m_port), database, username, password, ex);
-    if(!result &&
-      ex.get() && (ex->get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER))
+    try
     {
-      //It must be using a default password, so any failure would not be due to a wrong password.
-      //However, pg_ctl sometimes reports success before it is really ready to let us connect,
-      //so in this case we can just keep trying until it works, with a very long timeout.
-      count_retries++;
-      const guint max_retries = m_network_shared ? MAX_RETRIES_EVER : MAX_RETRIES_KNOWN_PASSWORD;
-      if(count_retries > max_retries)
+      result = attempt_connect(port_as_string(m_port), database, username, password);
+    }
+    catch(const ExceptionConnection& ex)
+    {
+      if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
       {
-        keep_trying = false;
+        //It must be using a default password, so any failure would not be due to a wrong password.
+        //However, pg_ctl sometimes reports success before it is really ready to let us connect,
+        //so in this case we can just keep trying until it works, with a very long timeout.
+        count_retries++;
+        const guint max_retries = m_network_shared ? MAX_RETRIES_EVER : MAX_RETRIES_KNOWN_PASSWORD;
+        if(count_retries > max_retries)
+        {
+          keep_trying = false;
+          continue;
+        }
+
+        std::cout << "debug: " << G_STRFUNC << ": Waiting and retrying the connection due to suspected too-early success of pg_ctl. retries=" << count_retries << ", max_retries=" << m_network_shared << std::endl;
+
+        //Wait:
+        Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
+          sigc::connection connection_timeout = Glib::signal_timeout().connect(
+          sigc::bind(sigc::ptr_fun(&on_timeout_delay), sigc::ref(mainloop)),
+          1000 /* 1 second */);
+        mainloop->run();
+        connection_timeout.disconnect();
+
+        keep_trying = true;
         continue;
       }
-
-      std::cout << "DEBUG: Glom::PostgresSelfHosted::connect(): Waiting and retrying the connection due to suspected too-early success of pg_ctl." << std::endl;
-
-      //Wait:
-      Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
-        sigc::connection connection_timeout = Glib::signal_timeout().connect(
-        sigc::bind(sigc::ptr_fun(&on_timeout_delay), sigc::ref(mainloop)),
-        1000 /* 1 second */);
-      mainloop->run();
-      connection_timeout.disconnect();
-
-      keep_trying = true;
-      continue;
+      else
+      {
+        throw ex;
+      }
     }
 
     keep_trying = false;
   }
 
-  error = ex;
+  //Save the connection details _only_ for later debug use:
+  
+  m_saved_database_name = database;
+  m_saved_username = username;
+  m_saved_password = password;
   return result;
 }
 
-bool PostgresSelfHosted::create_database(const Glib::ustring& database_name, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<Glib::Error>& error)
+bool PostgresSelfHosted::create_database(const Glib::ustring& database_name, const Glib::ustring& username, const Glib::ustring& password)
 {
-  return attempt_create_database(database_name, m_host, port_as_string(m_port), username, password, error);
+  return attempt_create_database(database_name, "localhost", port_as_string(m_port), username, password);
 }
 
 unsigned int PostgresSelfHosted::discover_first_free_port(unsigned int start_port, unsigned int end_port)
@@ -646,7 +674,7 @@ unsigned int PostgresSelfHosted::discover_first_free_port(unsigned int start_por
     std::cerr << "Create socket: " << WSAGetLastError() << std::endl;
 #else
     perror("Create socket");
-#endif
+#endif //G_OS_WIN32
     return 0;
   }
 
@@ -684,7 +712,7 @@ unsigned int PostgresSelfHosted::discover_first_free_port(unsigned int start_por
     }
     else
     {
-      //std::cout << "debug: ConnectionPool::discover_first_free_port(): port in use: " << port_to_try << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": port in use: " << port_to_try << std::endl;
     }
 
     if(available)
@@ -695,7 +723,7 @@ unsigned int PostgresSelfHosted::discover_first_free_port(unsigned int start_por
       close(fd);
       #endif //G_OS_WIN32
 
-      //std::cout << "debug: ConnectionPool::discover_first_free_port(): Found: returning " << port_to_try << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": Found: returning " << port_to_try << std::endl;
       return port_to_try;
     }
 
@@ -706,9 +734,9 @@ unsigned int PostgresSelfHosted::discover_first_free_port(unsigned int start_por
   closesocket(fd);
 #else
   close(fd);
-#endif
+#endif //G_OS_WIN32
 
-  std::cerr << "debug: ConnectionPool::discover_first_free_port(): No port was available." << std::endl;
+  std::cerr << G_STRFUNC << ": No port was available." << std::endl;
   return 0;
 }
 
