@@ -89,16 +89,11 @@ bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_tabl
   //TODO: This is duplicated in box_data_related_list.cc and box_data_portal.cc. Just use code from the base class?
   if(show_title)
   {
-    Glib::ustring relationship_title;
-    if(m_portal && m_portal->get_has_relationship_name())
-      relationship_title = m_portal->get_title_used(Glib::ustring() /* parent title - not relevant */);
-    else
-    {
-      //Note to translators: This text is shown instead of a table title, when the table has not yet been chosen.
-      relationship_title = _("Undefined Table");
-    }
+    Glib::ustring title;
+    if(m_portal)
+      title = m_portal->get_title();
 
-    m_Label.set_markup(Utils::bold_message(relationship_title));
+    m_Label.set_markup(Utils::bold_message(title));
     m_Label.show();
 
     m_Alignment.set_padding(Utils::DEFAULT_SPACING_SMALL /* top */, 0, Utils::DEFAULT_SPACING_LARGE /* left */, 0);
@@ -214,7 +209,7 @@ bool Box_Data_Calendar_Related::fill_from_database()
        continue;
 
       //Get the date value for this row:
-      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);
+      const Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);
       const Glib::Date date = value_date.get_date();
 
       //Get all the values for this row:
@@ -556,7 +551,7 @@ void Box_Data_Calendar_Related::on_calendar_button_press_event(GdkEventButton *e
 #endif
 
   GdkModifierType mods;
-  gdk_window_get_pointer( gtk_widget_get_window(Gtk::Widget::gobj()), 0, 0, &mods );
+  gdk_window_get_device_position( gtk_widget_get_window(Gtk::Widget::gobj()), event->device, 0, 0, &mods );
   if(mods & GDK_BUTTON3_MASK)
   {
     //Give user choices of actions on this item:
