@@ -123,7 +123,8 @@ protected:
   Gtk::Label* m_label_table_name;
   
   Gtk::Box* m_hbox_rows_count;
-  Gtk::SpinButton* m_spinbutton_rows_count;
+  Gtk::SpinButton* m_spinbutton_rows_count_min;
+  Gtk::SpinButton* m_spinbutton_rows_count_max;
 
   Glib::RefPtr<TreeStore_Layout> m_model_items;
 };
